package simple

import (
	"net/http"
	"io"
	"os"
	"path/filepath"
	"log"
	"encoding/json"
	"fmt"
	"github.com/blang/semver"
)

func rescanHandler(w http.ResponseWriter, r *http.Request) {
	distroName := r.URL.Query().Get("distro")

	if distroName == "" {
		distroName = "stable"
	}

	if _, exists := distros[distroName]; !exists {
		httpErrorf(w, "Unable to find distro %s", distroName)
		return
	}

	distro := &Distro{Name: distroName, Architectures: make(map[string]map[string]*PackageFile)}

	scanInitialPackages(conf, distro)

	distros[distroName] = distro
}

func uploadHandler(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		http.Error(w, "method not supported", http.StatusMethodNotAllowed)
		return
	}

	archType := r.URL.Query().Get("arch")

	if archType == "" {
		archType = "all"
	}

	distroName := r.URL.Query().Get("distro")

	if distroName == "" {
		distroName = "stable"
	}

	key := r.URL.Query().Get("key")

	if key == "" || key != conf.Http.Key {
		http.Error(w, "unauthorized", 403)
		return
	}

	force := false

	forceStr := r.URL.Query().Get("force")

	if forceStr != "" && forceStr == "true" {
		force = true
	}

	reader, err := r.MultipartReader()

	if err != nil {
		httpErrorf(w, "error creating multipart reader: %s", err)
		return
	}

	mutex.RLock()

	distro, exists := distros[distroName]

	if !exists {
		httpErrorf(w, "invalid distro: %s", distroName)
		mutex.RUnlock()
		return
	}

	packages, exists := distro.Architectures[archType]

	if !exists {
		httpErrorf(w, "invalid arch: %s", archType)
		mutex.RUnlock()
		return
	}

	mutex.RUnlock()

	// Lock to prevent concurrent modification
	mutex.Lock()
	defer mutex.Unlock()

	for {
		part, err := reader.NextPart()

		if err == io.EOF {
			break
		}

		if part.FileName() == "" {
			continue
		}
		newPath := conf.PoolPackagePath(distroName, archType, part.FileName())

		if _, err := os.Stat(newPath); err != nil {
			if os.IsNotExist(err) {
				if err := os.MkdirAll(newPath, 0755); err != nil {
					httpErrorf(w, "error creating path: %s", err)
					return
				}
			}
		}

		dst, err := os.Create(filepath.Join(newPath, part.FileName()))

		if err != nil {
			httpErrorf(w, "error creating deb file: %s", err)
			return
		}

		defer dst.Close()

		if _, err := io.Copy(dst, part); err != nil {
			httpErrorf(w, "error writing deb file: %s", err)
			return
		}

		// Get package name, if it already exists remove the old file.
		f, err := newPackageFile(newPath, part.FileName())

		if err != nil {
			httpErrorf(w, "error loading package info: %s", err)
		}

		if p, exists := packages[f.Info.Package]; exists {
			v1, err1 := semver.Parse(p.Info.Version)
			v2, err2 := semver.Parse(f.Info.Version)

			if err1 == nil && err2 == nil && v1.Compare(v2) > 0 && !force {
				// Don't replace newer package
				httpErrorf(w, "version in old package is greater than new: %s, %s - override with \"force\"", p.Info.Version, f.Info.Version)
				return
			}

			// Archive old file
			log.Println("Replacing", p.Name, "with", f.Name)

			if err := os.Remove(p.Path); err != nil {
				httpErrorf(w, "Unable to remove old package: %s", err)
				return
			}
		}

		packages[f.Info.Package] = f
	}

	log.Println("got lock, updating package list...")

	// Recreate the package index and release file.

	if err := createPackagesCached(conf, distroName, archType, packages); err != nil {
		httpErrorf(w, "error creating package: %s", err)
		return
	}

	err = createRelease(conf, distroName, archType)

	if err != nil {
		httpErrorf(w, "error creating package: %s", err)
		return
	}

	w.WriteHeader(http.StatusCreated)
}

func deleteHandler(w http.ResponseWriter, r *http.Request) {
	if r.Method != "DELETE" {
		http.Error(w, "method not supported", http.StatusMethodNotAllowed)
		return
	}
	var req DeleteObj

	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
		httpErrorf(w, "failed to decode json: %s", err)
		return
	}

	mutex.RLock()

	distro, exists := distros[req.DistroName]

	if !exists {
		httpErrorf(w, "invalid distro: %s", req.DistroName)
		mutex.RUnlock()
		return
	}

	packages, exists := distro.Architectures[req.Arch]

	if !exists {
		httpErrorf(w, "invalid arch: %s", req.Arch)
		mutex.RUnlock()
		return
	}

	mutex.RUnlock()

	key := r.URL.Query().Get("key")

	if key == "" || key != conf.Http.Key {
		http.Error(w, "unauthorized", http.StatusForbidden)
		return
	}

	debPath := filepath.Join(conf.ArchPath(req.DistroName, req.Arch), req.Filename)

	if err := os.Remove(debPath); err != nil {
		httpErrorf(w, "failed to delete: %s", err)
		return
	}

	mutex.Lock()
	defer mutex.Unlock()

	if err := createPackagesCached(conf, req.DistroName, req.Arch, packages); err != nil {
		httpErrorf(w, "failed to delete package: %s", err)
		return
	}

	if err := createRelease(conf, req.DistroName, req.Arch); err != nil {
		httpErrorf(w, "failed to delete package: %s", err)
		return
	}

	w.WriteHeader(http.StatusOK)
}

func httpErrorf(w http.ResponseWriter, format string, a ...interface{}) {
	err := fmt.Errorf(format, a...)
	log.Println(err)
	http.Error(w, err.Error(), http.StatusInternalServerError)
}